@@ -14,13 +14,14 @@
 # limitations under the License.
 
 import datetime
-import sys
 import os
-from fabric.api import (hide, local, env, execute, abort, lcd, sudo, run, cd, puts, settings as fab_settings, quiet)
+import sys
+from fabric.api import (hide, local, env, execute, abort, lcd, sudo, run, cd, puts, settings as fab_settings,
+                        quiet, prompt, get)
 from fabric.colors import red, green, yellow, cyan
 from fabric.contrib import django as fab_django
 from fabric.contrib.console import confirm
-from fabric.contrib.files import exists as remote_exists
+from fabric.contrib.files import exists as remote_exists, append as remote_append
 from fabric.decorators import task, roles
 from fabric.state import output as output_level
 from fabric.utils import error
@@ -92,9 +93,87 @@ def deploy_setting(project_name=None, deploy_settings_path=None, **setting_patch
     # Load project_name
     env.project_name = project_name
 
+
+def load_web_servers():
+    if len(env.roledefs['web_servers']) == 0:
+        with hide('everything'):
+            execute('server.web_server', *env.deploy_settings.get('WEB_SERVERS'))
+            verbose = env.deploy_settings.get('VERBOSE')
+            if verbose != 0:
+                puts('')
+    if len(env.roledefs['web_servers']) == 0:
+        abort('Where do you want to deploy to?')
+
 # Tasks
 
 
+@task
+def add_user(role='web_servers', project_name=None, deploy_settings_path=None, **additional_settings):
+    """Add current user to remote server
+    """
+    ssh_key_path = prompt('Path of your ssh public key: ', default='~/.ssh/id_rsa.pub')
+    ssh_key_path = os.path.abspath(os.path.expanduser(ssh_key_path))
+
+    if not os.path.exists(ssh_key_path):
+        abort('No file found at %s' % ssh_key_path)
+
+    execute(
+        deploy_setting, project_name=project_name, deploy_settings_path=deploy_settings_path, **additional_settings
+    )
+    execute(load_web_servers)
+
+    ssh_home_path = '~/.ssh'
+    ssh_authorized_keys_path = '%s/authorized_keys' % ssh_home_path
+    with open(ssh_key_path, 'r') as f:
+        ssh_key = f.read().strip()
+
+    print ''
+
+    @roles(role)
+    def core():
+        # Create ssh home
+        if not remote_exists(ssh_home_path):
+            run('mkdir -p %s' % ssh_home_path)
+            run('chmod 700 %s' % ssh_home_path)
+        if not remote_exists(ssh_authorized_keys_path):
+            run('touch %s' % ssh_authorized_keys_path)
+            run('chmod 644 %s' % ssh_authorized_keys_path)
+
+        with temporary_directory(prefix='add_deploy_user~%s~' % project_name) as tmp_directory:
+            local_authorized_keys_path = os.path.join(tmp_directory, 'remote_authorized_keys')
+            # Get remote key
+            with hide('everything'):
+                get(ssh_authorized_keys_path, local_authorized_keys_path)
+            with open(local_authorized_keys_path, 'r') as local_authorized_keys:
+                for authorized_key in local_authorized_keys:
+                    if authorized_key.strip() == ssh_key:
+                        print cyan('You have been here.')
+                        return
+
+        remote_append(ssh_authorized_keys_path, ssh_key)
+        print green('You have joined here.')
+
+    with fab_settings():
+        execute(core)
+
+
+@task
+def setup_server():
+    if env.host_string is None:
+        abort('Assign a target server you can reach')
+
+    # Do you have user "eggplant"?
+    with fab_settings(hide('everything'), warn_only=True):
+        has_eggplant_user = run('egrep ^eggplant: /etc/passwd').return_code == 0
+    if not has_eggplant_user:
+        print cyan('Create user ') + cyan('"eggplant"', bold=True)
+        sudo('useradd eggplant')
+        sudo('passwd eggplant')
+        sudo('usermod -a -G wheel eggplant')
+
+    execute(add_user)
+
+
 @task(default=True)
 def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy', **additional_settings):
     """Deploy current django project to targeted web servers
@@ -151,13 +230,7 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy', **
     env.use_ssh_config = deploy_settings.get('USE_SSH_CONFIG')
 
     # Load web servers
-    if len(env.roledefs['web_servers']) == 0:
-        with hide('everything'):
-            execute('server.web_server', *deploy_settings.get('WEB_SERVERS'))
-            if verbose != 0:
-                puts('')
-    if len(env.roledefs['web_servers']) == 0:
-        abort('Where do you want to deploy to?')
+    execute(load_web_servers)
 
     with temporary_directory(prefix='deploy~%s~' % project_name) as tmp_directory:
         env.tmp_directory = tmp_directory
@@ -396,10 +469,6 @@ def deploy_to_remote():
         # file existence
         target_file_existence = remote_exists
 
-    # Get env
-    with hide('everything'):
-        remote_user = target_run('whoami', **capture_output)
-        remote_group = target_run('id -g -n %s' % remote_user, **capture_output)
     # Get eggplant settings in server
     eggplant_server_setting = {}
     with fab_settings(hide('everything'), warn_only=True):
@@ -439,7 +508,7 @@ def deploy_to_remote():
         # Make room
         target_sudo('mkdir -p %s' % remote_project_path)
         target_sudo('mkdir -p %s' % remote_project_log_root)
-        target_sudo('chown -R %s:%s %s' % (remote_user, remote_group, remote_project_path))
+        target_sudo('chown -R eggplant:eggplant %s' % remote_project_path)
         # Source folder with git
         target_run('mkdir -p %s' % remote_project_source_root)
         with target_cd(remote_project_path):
@@ -517,9 +586,13 @@ def deploy_to_remote():
     # Clean old pyc
     target_run('find %s -name \'*.pyc\' -exec rm -rf {} \;' % remote_project_source_root)
     # Do rsync
-    rsync_argument = '-DLprtz --delete --delete-excluded --exclude \'*.pyc\' --exclude-from \'.gitignore\''
-    if verbose == 3:
+    rsync_argument = '-DLhrctz --delete --delete-excluded --exclude \'*.pyc\' --exclude-from \'.gitignore\''
+    if verbose == 0:
+        rsync_argument += ' -q'
+    elif verbose == 2:
         rsync_argument += ' -v'
+    elif verbose == 3:
+        rsync_argument += ' -Pv'
     local(rsync(
         project_src_root + ('' if project_src_root.endswith('/') else '/'),
         (host_string + ':' if is_remote else '') + remote_project_source_root, rsync_argument=rsync_argument))
