@@ -48,11 +48,13 @@ def web_server(*servers):
         except (ValueError, AddrFormatError):
             hosts += [server]
 
+    user_hosts = []
     for host in hosts:
         puts('Add "%s" to web server list' % host)
+        user_hosts.append('eggplant@%s' % host)
 
     env.roledefs.update({
-        'web_servers': env.roledefs['web_servers'] + hosts
+        'web_servers': env.roledefs['web_servers'] + user_hosts
     })
 
 
